@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
-import pageadmin
-import useradmin
-import permissionadmin
-
+import cms.admin.pageadmin
+import cms.admin.useradmin
+import cms.admin.permissionadmin
+import cms.admin.settingsadmin
+import cms.admin.static_placeholder  # nopyflakes
 # Piggyback off admin.autodiscover() to discover cms plugins
 from cms import plugin_pool
 from cms.apphook_pool import apphook_pool
 plugin_pool.plugin_pool.discover_plugins()
 
-apphook_pool.discover_apps()
\ No newline at end of file
+apphook_pool.discover_apps()
