@@ -238,6 +238,14 @@ class PageAdmin(PlaceholderAdminMixin, ModelAdmin):
         if new and Page.objects.filter(site_id=obj.site_id).count() == 1:
             obj.publish(language)
 
+    def get_fieldsets(self, request, obj=None):
+        form = self.get_form(request, obj, fields=None)
+        if getattr(form, 'fieldsets', None) is None:
+            fields = list(form.base_fields) + list(self.get_readonly_fields(request, obj))
+            return [(None, {'fields': fields})]
+        else:
+            return form.fieldsets
+
     def get_form(self, request, obj=None, **kwargs):
         """
         Get PageForm for the Page model and modify its fields depending on
@@ -295,8 +303,11 @@ class PageAdmin(PlaceholderAdminMixin, ModelAdmin):
             self.inlines = []
             for name in ['slug', 'title']:
                 form.base_fields[name].initial = u''
-            if 'target' in request.GET:
-                target = request.GET['target']
+            if 'target' in request.GET or 'copy_target' in request.GET:
+                if 'target' in request.GET:
+                    target = request.GET['target']
+                if 'copy_target' in request.GET:
+                    target = request.GET['copy_target']
                 if 'position' in request.GET:
                     position = request.GET['position']
                     if position == 'last-child' or position == 'first-child':
