@@ -1,10 +1,7 @@
 # -*- coding: utf-8 -*-
-from datetime import datetime
-from cms.utils.conf import get_cms_setting
 from django.db.models import Q
 from django.contrib.sites.models import Site
 from cms.publisher.query import PublisherQuerySet
-from django.conf import settings
 from cms.exceptions import NoHomeFound
 from django.utils import timezone
 
@@ -48,21 +45,20 @@ class PageQuerySet(PublisherQuerySet):
         else:
             return self.exclude(id__in=exclude_list)
 
-    def published(self, site=None):
-        pub = self.on_site(site).filter(published=True)
+    def published(self, language=None, site=None):
 
-        if get_cms_setting('SHOW_START_DATE'):
-            pub = pub.filter(
-                Q(publication_date__lt=timezone.now()) |
-                Q(publication_date__isnull=True)
+        if language:
+            pub = self.on_site(site).filter(
+                Q(publication_date__lte=timezone.now()) | Q(publication_date__isnull=True),
+                Q(publication_end_date__gt=timezone.now()) | Q(publication_end_date__isnull=True),
+                title_set__published=True, title_set__language=language
             )
-
-        if get_cms_setting('SHOW_END_DATE'):
-            pub = pub.filter(
-                Q(publication_end_date__gte=timezone.now()) |
-                Q(publication_end_date__isnull=True)
+        else:
+            pub = self.on_site(site).filter(
+                Q(publication_date__lte=timezone.now()) | Q(publication_date__isnull=True),
+                Q(publication_end_date__gt=timezone.now()) | Q(publication_end_date__isnull=True),
+                title_set__published=True
             )
-
         return pub
 
     def expired(self):
@@ -78,7 +74,7 @@ class PageQuerySet(PublisherQuerySet):
 
     def get_home(self, site=None):
         try:
-            home = self.published(site).all_root().order_by("tree_id")[0]
+            home = self.published(site=site).all_root().order_by("tree_id")[0]
         except IndexError:
             raise NoHomeFound('No Root page found. Publish at least one page!')
         return home
