@@ -1,18 +1,19 @@
 # -*- coding: utf-8 -*-
 from cms.models.placeholdermodel import Placeholder
-from cms.plugin_processors import (plugin_meta_context_processor,
-    mark_safe_plugin_processor)
+from cms.plugin_processors import (plugin_meta_context_processor, mark_safe_plugin_processor)
 from cms.utils import get_language_from_request
+from cms.utils.compat.type_checks import string_types
 from cms.utils.conf import get_cms_setting
 from cms.utils.django_load import iterload_objects
-from cms.utils.placeholder import get_placeholder_conf
-from django.conf import settings
+from cms.utils.placeholder import get_placeholder_conf, restore_sekizai_context
 from django.template import Template, Context
-from django.template.defaultfilters import title
 from django.template.loader import render_to_string
-from django.utils.translation import ugettext_lazy as _
+from django.utils.safestring import mark_safe
+
 
 # these are always called before all other plugin context processors
+from sekizai.helpers import Watcher
+
 DEFAULT_PLUGIN_CONTEXT_PROCESSORS = (
     plugin_meta_context_processor,
 )
@@ -30,26 +31,27 @@ class PluginContext(Context):
     Additional processors can be specified as a list of callables
     using the "processors" keyword argument.
     """
+
     def __init__(self, dict, instance, placeholder, processors=None, current_app=None):
         super(PluginContext, self).__init__(dict, current_app=current_app)
         if not processors:
             processors = []
         for processor in DEFAULT_PLUGIN_CONTEXT_PROCESSORS:
-            self.update(processor(instance, placeholder))
+            self.update(processor(instance, placeholder, self))
         for processor in iterload_objects(get_cms_setting('PLUGIN_CONTEXT_PROCESSORS')):
-            self.update(processor(instance, placeholder))
+            self.update(processor(instance, placeholder, self))
         for processor in processors:
-            self.update(processor(instance, placeholder))
+            self.update(processor(instance, placeholder, self))
 
-def render_plugin(context, instance, placeholder, template, processors=None,
-                  current_app=None):
+
+def render_plugin(context, instance, placeholder, template, processors=None, current_app=None):
     """
     Renders a single plugin and applies the post processors to it's rendered
     content.
     """
     if not processors:
         processors = []
-    if isinstance(template, basestring):
+    if isinstance(template, string_types):
         content = render_to_string(template, context_instance=context)
     elif isinstance(template, Template):
         content = template.render(context)
@@ -63,12 +65,13 @@ def render_plugin(context, instance, placeholder, template, processors=None,
         content = processor(instance, placeholder, content, context)
     return content
 
+
 def render_plugins(plugins, context, placeholder, processors=None):
     """
     Renders a collection of plugins with the given context, using the appropriate processors
     for a given placeholder name, and returns a list containing a "rendered content" string
     for each plugin.
-    
+
     This is the main plugin rendering utility function, use this function rather than
     Plugin.render_plugin().
     """
@@ -82,82 +85,123 @@ def render_plugins(plugins, context, placeholder, processors=None):
         context.pop()
     return out
 
-def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder", lang=None):
+
+def render_placeholder(placeholder, context_to_copy, name_fallback="Placeholder", lang=None, default=None):
     """
-    Renders plugins for a placeholder on the given page using shallow copies of the 
+    Renders plugins for a placeholder on the given page using shallow copies of the
     given context, and returns a string containing the rendered output.
     """
-    from cms.plugins.utils import get_plugins
+    if not placeholder:
+        return
+    from cms.utils.plugins import get_plugins
     context = context_to_copy
     context.push()
     request = context['request']
-    plugins = [plugin for plugin in get_plugins(request, placeholder, lang=lang)]
+    if not hasattr(request, 'placeholder'):
+        request.placeholders = []
+    request.placeholders.append(placeholder)
+    if hasattr(placeholder, 'content_cache'):
+        return mark_safe(placeholder.content_cache)
     page = placeholder.page if placeholder else None
+    # It's kind of duplicate of the similar call in `get_plugins`, but it's required
+    # to have a valid language in this function for `get_fallback_languages` to work
+    if lang:
+        save_language = lang
+    else:
+        lang = get_language_from_request(request)
+        save_language = lang
+
+    # Prepend frontedit toolbar output if applicable
+    edit = False
+    toolbar = getattr(request, 'toolbar', None)
+
+    if getattr(toolbar, 'edit_mode', False):
+        edit = True
+    if edit:
+        from cms.middleware.toolbar import toolbar_plugin_processor
+
+        processors = (toolbar_plugin_processor,)
+    else:
+        processors = None
+    from django.core.cache import cache
+    if get_cms_setting('PLACEHOLDER_CACHE'):
+        cache_key = placeholder.get_cache_key(lang)
+        if not edit and placeholder and not hasattr(placeholder, 'cache_checked'):
+            cached_value = cache.get(cache_key)
+            if not cached_value is None:
+                restore_sekizai_context(context, cached_value['sekizai'])
+                return mark_safe(cached_value['content'])
     if page:
         template = page.template
     else:
         template = None
+
+    plugins = [plugin for plugin in get_plugins(request, placeholder, template, lang=lang)]
+
     # Add extra context as defined in settings, but do not overwrite existing context variables,
     # since settings are general and database/template are specific
-    # TODO this should actually happen as a plugin context processor, but these currently overwrite 
+    # TODO this should actually happen as a plugin context processor, but these currently overwrite
     # existing context -- maybe change this order?
     slot = getattr(placeholder, 'slot', None)
     extra_context = {}
     if slot:
         extra_context = get_placeholder_conf("extra_context", slot, template, {})
     for key, value in extra_context.items():
-        if not key in context:
+        if key not in context:
             context[key] = value
 
     content = []
-
-    # Prepend frontedit toolbar output if applicable
-    edit = False
-    toolbar = getattr(request, 'toolbar', None)
-
-    if (getattr(toolbar, 'edit_mode', False) and
-        (not page or page.has_change_permission(request))):
-            edit = True
-    if edit:
-        from cms.middleware.toolbar import toolbar_plugin_processor
-        processors = (toolbar_plugin_processor,)
-    else:
-        processors = None
-
+    watcher = Watcher(context)
     content.extend(render_plugins(plugins, context, placeholder, processors))
-    content = "".join(content)
-    if edit:
-        content = render_placeholder_toolbar(placeholder, context, content, name_fallback)
-    context.pop()
-    return content
+    toolbar_content = ''
 
-def render_placeholder_toolbar(placeholder, context, content, name_fallback=None):
+    if edit:
+        if not hasattr(request.toolbar, 'placeholders'):
+            request.toolbar.placeholders = {}
+        if placeholder.pk not in request.toolbar.placeholders:
+            request.toolbar.placeholders[placeholder.pk] = placeholder
+    if edit:
+        toolbar_content = mark_safe(render_placeholder_toolbar(placeholder, context, name_fallback, save_language))
+    if content:
+        content = mark_safe("".join(content))
+    elif default:
+        #should be nodelist from a template
+        content = mark_safe(default.render(context_to_copy))
+    else:
+        content = ''
+    context['content'] = content
+    context['placeholder'] = toolbar_content
+    context['edit'] = edit
+    result = render_to_string("cms/toolbar/content.html", context)
+    changes = watcher.get_changes()
+    if placeholder and not edit and placeholder.cache_placeholder and get_cms_setting('PLACEHOLDER_CACHE'):
+        cache.set(cache_key, {'content': result, 'sekizai': changes}, get_cms_setting('CACHE_DURATIONS')['content'])
+    context.pop()
+    return result
+
+
+def render_placeholder_toolbar(placeholder, context, name_fallback, save_language):
     from cms.plugin_pool import plugin_pool
     request = context['request']
     page = placeholder.page if placeholder else None
     if not page:
         page = getattr(request, 'current_page', None)
     if page:
-        template = page.template
         if name_fallback and not placeholder:
             placeholder = Placeholder.objects.create(slot=name_fallback)
             page.placeholders.add(placeholder)
             placeholder.page = page
-    else:
-        template = None
     if placeholder:
         slot = placeholder.slot
     else:
         slot = None
-    installed_plugins = plugin_pool.get_all_plugins(slot, page)
-    name = get_placeholder_conf("name", slot, template, title(slot))
-    name = _(name)
     context.push()
-    context['installed_plugins'] = installed_plugins
-    context['language'] = get_language_from_request(request)
-    context['placeholder_label'] = name
+
+    # to restrict child-only plugins from draggables..
+    context['allowed_plugins'] = [cls.__name__ for cls in plugin_pool.get_all_plugins(slot, page)]
     context['placeholder'] = placeholder
+    context['language'] = save_language
     context['page'] = page
     toolbar = render_to_string("cms/toolbar/placeholder.html", context)
     context.pop()
-    return "".join([toolbar, content])
+    return toolbar
