@@ -1,4 +1,4 @@
-from django.conf.urls.defaults import *
+from django.conf.urls import patterns, url, include
 from django.utils.translation import ugettext_lazy as _
 
 """
@@ -14,5 +14,5 @@ urlpatterns = patterns('cms.test_utils.project.sampleapp.views',
     url(r'^category/(?P<id>[0-9]+)/$', 'category_view', name='category_view'),
     url(r'^notfound/$', 'notfound', name='notfound'),
     url(r'^extra_1/$', 'extra_view', {'message': 'test urlconf'}, name='extra_first'),
-    url(r'^', include('cms.test_utils.project.sampleapp.urls_extra')),
+    url(r'^', include('cms.test_utils.project.sampleapp.urls_extra'), {'opts': 'someopts'}),
 )
