@@ -1,7 +1,7 @@
+from django.conf.urls import include, url, patterns
 from cms.apphook_pool import apphook_pool
 from cms.views import details
 from django.conf import settings
-from django.conf.urls.defaults import url, patterns
 
 if settings.APPEND_SLASH:
     reg = url(r'^(?P<slug>[0-9A-Za-z-_.//]+)/$', details, name='pages-details-by-slug')
@@ -10,7 +10,12 @@ else:
 
 urlpatterns = [
     # Public pages
-    url(r'^$', details, {'slug':''}, name='pages-root'),
+    url(r'^example/',
+        include('cms.test_utils.project.sampleapp.urls_example', namespace="example1", app_name='example_app')),
+    url(r'^example2/',
+        include('cms.test_utils.project.sampleapp.urls_example', namespace="example2", app_name='example_app')),
+    url(r'^$', details, {'slug': ''}, name='pages-root'),
+
     reg,
 ]
 
@@ -20,6 +25,12 @@ if apphook_pool.get_apphooks():
     """
     from cms.appresolver import get_app_patterns
     urlpatterns = get_app_patterns() + urlpatterns
-    #urlpatterns = (dynamic_app_regex_url_resolver, ) + urlpatterns
-    
+
 urlpatterns = patterns('', *urlpatterns)
+
+
+if settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS:
+    import debug_toolbar
+    urlpatterns += patterns('',
+        url(r'^__debug__/', include(debug_toolbar.urls)),
+    )
