@@ -1,7 +1,6 @@
 from cms.utils.conf import get_cms_setting
 from django.conf import settings
-from django.conf.urls.defaults import handler500, handler404, patterns, include, \
-    url
+from django.conf.urls import patterns, include, url
 try:
     from django.conf.urls.i18n import i18n_patterns
 except ImportError:
@@ -23,3 +22,10 @@ urlpatterns = patterns('',
 urlpatterns += i18n_patterns('',
     url(r'^', include('cms.test_utils.project.cms_urls_for_apphook_tests')),
 )
+
+
+if settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS:
+    import debug_toolbar
+    urlpatterns += patterns('',
+        url(r'^__debug__/', include(debug_toolbar.urls)),
+    )
