@@ -1,14 +1,9 @@
 from cms.utils.conf import get_cms_setting
 from django.conf import settings
-from django.conf.urls.defaults import patterns, include, \
+from django.conf.urls import patterns, include, \
     url
 from django.contrib import admin
 
-try:
-    from django.conf.urls.i18n import i18n_patterns
-except ImportError:
-    from i18nurls.i18n import i18n_patterns
-
 from django.contrib.staticfiles.urls import staticfiles_urlpatterns
 
 admin.autodiscover()
@@ -30,3 +25,10 @@ urlpatterns += patterns('',
                         url(r'^example/$', 'cms.test_utils.project.placeholderapp.views.example_view'),
                         url(r'^', include('cms.urls')),
 )
+
+
+if settings.DEBUG and 'debug_toolbar' in settings.INSTALLED_APPS:
+    import debug_toolbar
+    urlpatterns += patterns('',
+        url(r'^__debug__/', include(debug_toolbar.urls)),
+    )
