@@ -1,28 +1,27 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
 import copy
-from django.test import Client
+
+from django.contrib.sites.models import Site
+
 from cms.api import create_page
-from cms.models import Page
+from cms.models import Page, Placeholder
 from cms.utils import get_cms_setting
-from cms.views import details
 from cms.test_utils.testcases import CMSTestCase
 from cms.test_utils.util.context_managers import SettingsOverride
-from django.contrib.auth.models import User
-from django.contrib.sites.models import Site
+from django.core.urlresolvers import reverse
+
 
 class SiteTestCase(CMSTestCase):
     """Site framework specific test cases.
     
     All stuff which is changing settings.SITE_ID for tests should come here.
     """
+
     def setUp(self):
         self.assertEqual(Site.objects.all().count(), 1)
         with SettingsOverride(SITE_ID=1):
-
-            u = User(username="test", is_staff=True, is_active=True, is_superuser=True)
-            u.set_password("test")
-            u.save()
+            u = self._create_user("test", True, True)
 
             # setup sites
             self.site2 = Site.objects.create(domain="sample2.com", name="sample2.com", pk=2)
@@ -34,7 +33,6 @@ class SiteTestCase(CMSTestCase):
     def tearDown(self):
         self._login_context.__exit__(None, None, None)
 
-
     def test_site_framework(self):
         #Test the site framework, and test if it's possible to disable it
         with SettingsOverride(SITE_ID=self.site2.pk):
@@ -57,9 +55,16 @@ class SiteTestCase(CMSTestCase):
             # without param
             self.assertEqual(Page.objects.drafts().on_site().count(), 1)
 
+    def test_site_preview(self):
+        page = create_page("page", "nav_playground.html", "de", site=self.site2, published=True)
+        with self.login_user_context(self.get_superuser()):
+            response = self.client.get(reverse('admin:cms_page_preview_page', args=[page.pk, 'de']))
+            self.assertEqual(response.status_code, 302)
+            self.assertEqual(response._headers['location'][1], 'http://sample2.com/de/?edit&language=de')
+
     def test_site_publish(self):
         self._login_context.__exit__(None, None, None)
-        pages = {"2": range(0, 5), "3": range(0, 5)}
+        pages = {"2": list(range(0, 5)), "3": list(range(0, 5))}
         lang_settings = copy.deepcopy(get_cms_setting('LANGUAGES'))
         lang_settings[3][1]['public'] = True
 
@@ -67,7 +72,7 @@ class SiteTestCase(CMSTestCase):
             with SettingsOverride(SITE_ID=self.site2.pk):
                 pages["2"][0] = create_page("page_2", "nav_playground.html", "de",
                                             site=self.site2)
-                pages["2"][0].publish()
+                pages["2"][0].publish('de')
                 pages["2"][1] = create_page("page_2_1", "nav_playground.html", "de",
                                             parent=pages["2"][0], site=self.site2)
                 pages["2"][2] = create_page("page_2_2", "nav_playground.html", "de",
@@ -78,9 +83,9 @@ class SiteTestCase(CMSTestCase):
                                             parent=pages["2"][1], site=self.site2)
 
                 for page in pages["2"]:
-                    page.publish()
+                    page.publish('de')
                 for page in pages["2"]:
-                    if page.is_home():
+                    if page.is_home:
                         page_url = "/de/"
                     else:
                         page_url = page.get_absolute_url(language='de')
@@ -90,7 +95,7 @@ class SiteTestCase(CMSTestCase):
             with SettingsOverride(SITE_ID=self.site3.pk):
                 pages["3"][0] = create_page("page_3", "nav_playground.html", "de",
                                             site=self.site3)
-                pages["3"][0].publish()
+                pages["3"][0].publish('de')
                 pages["3"][1] = create_page("page_3_1", "nav_playground.html", "de",
                                             parent=pages["3"][0], site=self.site3)
                 pages["3"][2] = create_page("page_3_2", "nav_playground.html", "de",
@@ -101,11 +106,18 @@ class SiteTestCase(CMSTestCase):
                                             parent=pages["3"][1], site=self.site3)
 
                 for page in pages["3"]:
-                    page.publish()
+                    page.publish('de')
                 for page in pages["3"]:
-                    if page.is_home():
+                    if page.is_home:
                         page_url = "/de/"
                     else:
                         page_url = page.get_absolute_url(language='de')
                     response = self.client.get(page_url)
-                    self.assertEqual(response.status_code, 200)
\ No newline at end of file
+                    self.assertEqual(response.status_code, 200)
+
+    def test_site_delete(self):
+        with SettingsOverride(SITE_ID=self.site2.pk):
+            create_page("page_2a", "nav_playground.html", "de", site=self.site2)
+            self.assertEqual(Placeholder.objects.count(), 2)
+            self.site2.delete()
+            self.assertEqual(Placeholder.objects.count(), 0)
