@@ -1,25 +1,29 @@
 from __future__ import with_statement
 import copy
-from django.test import RequestFactory, TestCase
 import os
-from cms.api import create_page, create_title, add_plugin
-from cms.models.pagemodel import Page, Placeholder
-from cms.plugins.text.cms_plugins import TextPlugin
-from cms.templatetags.cms_tags import (get_site_id, _get_page_by_untyped_arg,
-    _show_placeholder_for_page)
-from cms.test_utils.fixtures.templatetags import TwoPagesFixture
-from cms.test_utils.testcases import SettingsOverrideTestCase
-from cms.test_utils.util.context_managers import SettingsOverride
-from cms.utils import get_cms_setting
-from cms.utils.plugins import get_placeholders
+
+from django.contrib.auth.models import AnonymousUser
 from django.contrib.sites.models import Site
 from django.core import mail
 from django.core.exceptions import ImproperlyConfigured
 from django.http import HttpRequest
 from django.template import RequestContext, Context
+from django.test import RequestFactory, TestCase
 from django.template.base import Template
 from django.utils.html import escape
-from django.contrib.auth.models import User
+from djangocms_text_ckeditor.cms_plugins import TextPlugin
+
+from cms.middleware.toolbar import ToolbarMiddleware
+from cms.toolbar.toolbar import CMSToolbar
+from cms.api import create_page, create_title, add_plugin
+from cms.compat import get_user_model
+from cms.models.pagemodel import Page, Placeholder
+from cms.templatetags.cms_tags import _get_page_by_untyped_arg, _show_placeholder_for_page, _get_placeholder
+from cms.test_utils.fixtures.templatetags import TwoPagesFixture
+from cms.test_utils.testcases import SettingsOverrideTestCase, CMSTestCase
+from cms.test_utils.util.context_managers import SettingsOverride
+from cms.utils import get_cms_setting, get_site_id
+from cms.utils.plugins import get_placeholders
 
 
 class TemplatetagTests(TestCase):
@@ -82,10 +86,11 @@ class TemplatetagDatabaseTests(TwoPagesFixture, SettingsOverrideTestCase):
         control = self._getfirst()
         request = self.get_request('/')
         request.GET = {"edit": ''}
-        user = User(username="admin", password="admin", is_superuser=True, is_staff=True, is_active=True)
-        user.save()
+        user = self._create_user("admin", True, True)
         request.current_page = control
         request.user = user
+        middleware = ToolbarMiddleware()
+        middleware.process_request(request)
         page = _get_page_by_untyped_arg(control.pk, request, 1)
         self.assertEqual(page, control.publisher_draft)
 
@@ -140,6 +145,8 @@ class TemplatetagDatabaseTests(TwoPagesFixture, SettingsOverrideTestCase):
         Verify ``show_placeholder`` correctly handles being given an
         invalid identifier.
         """
+        User = get_user_model()
+
         with SettingsOverride(DEBUG=True):
             request = HttpRequest()
             request.REQUEST = {}
@@ -166,11 +173,13 @@ class TemplatetagDatabaseTests(TwoPagesFixture, SettingsOverrideTestCase):
         page_1 = create_page('Page 1', 'nav_playground.html', 'en', published=True,
                              in_navigation=True, reverse_id='page1')
         create_title("de", "Seite 1", page_1, slug="seite-1")
-        page_1.publish()
+        page_1.publish('en')
+        page_1.publish('de')
         page_2 = create_page('Page 2', 'nav_playground.html', 'en', page_1, published=True,
                              in_navigation=True, reverse_id='page2')
         create_title("de", "Seite 2", page_2, slug="seite-2")
-        page_2.publish()
+        page_2.publish('en')
+        page_2.publish('de')
         page_3 = create_page('Page 3', 'nav_playground.html', 'en', page_2, published=True,
                              in_navigation=True, reverse_id='page3')
         tpl = Template("{% load menu_tags %}{% page_language_url 'de' %}")
@@ -205,20 +214,41 @@ class TemplatetagDatabaseTests(TwoPagesFixture, SettingsOverrideTestCase):
             res = tpl.render(context)
             self.assertEqual(res, "/de/")
 
+    def test_create_placeholder_if_not_exist_in_template(self):
+        """
+        Tests that adding a new placeholder to a an exising page's template
+        creates the placeholder.
+        """
+        page = create_page('Test', 'col_two.html', 'en')
+        # I need to make it seem like the user added another plcaeholder to the SAME template.
+        page._template_cache = 'col_three.html'
 
-class NoFixtureDatabaseTemplateTagTests(TestCase):
+        class FakeRequest(object):
+            current_page = page
+            REQUEST = {'language': 'en'}
+
+        placeholder = _get_placeholder(page, page, dict(request=FakeRequest()), 'col_right')
+        page.placeholders.get(slot='col_right')
+        self.assertEqual(placeholder.slot, 'col_right')
+
+
+class NoFixtureDatabaseTemplateTagTests(CMSTestCase):
     def test_cached_show_placeholder_sekizai(self):
         from django.core.cache import cache
 
         cache.clear()
         from cms.test_utils import project
 
+        User = get_user_model()
+
         template_dir = os.path.join(os.path.dirname(project.__file__), 'templates', 'alt_plugin_templates',
                                     'show_placeholder')
         page = create_page('Test', 'col_two.html', 'en')
         placeholder = page.placeholders.all()[0]
         add_plugin(placeholder, TextPlugin, 'en', body='HIDDEN')
         request = RequestFactory().get('/')
+        request.user = User()
+        request.current_page = page
         with SettingsOverride(TEMPLATE_DIRS=[template_dir]):
             template = Template(
                 "{% load cms_tags sekizai_tags %}{% show_placeholder slot page 'en' 1 %}{% render_block 'js' %}")
@@ -237,6 +267,8 @@ class NoFixtureDatabaseTemplateTagTests(TestCase):
         placeholder = page.placeholders.all()[0]
         add_plugin(placeholder, TextPlugin, 'en', body='<b>Test</b>')
         request = RequestFactory().get('/')
+        request.user = AnonymousUser()
+        request.current_page = page
         template = Template(
             "{% load cms_tags sekizai_tags %}{% show_placeholder slot page 'en' 1 %}{% render_block 'js' %}")
         context = RequestContext(request, {'page': page, 'slot': placeholder.slot})
@@ -250,12 +282,14 @@ class NoFixtureDatabaseTemplateTagTests(TestCase):
 
     def test_cached_show_placeholder_preview(self):
         from django.core.cache import cache
-
         cache.clear()
-        page = create_page('Test', 'col_two.html', 'en')
+        page = create_page('Test', 'col_two.html', 'en', published=True)
         placeholder = page.placeholders.all()[0]
         add_plugin(placeholder, TextPlugin, 'en', body='<b>Test</b>')
         request = RequestFactory().get('/')
+        user = self._create_user("admin", True, True)
+        request.current_page = page.publisher_public
+        request.user = user
         template = Template(
             "{% load cms_tags %}{% show_placeholder slot page 'en' 1 %}")
         context = RequestContext(request, {'page': page, 'slot': placeholder.slot})
@@ -264,7 +298,45 @@ class NoFixtureDatabaseTemplateTagTests(TestCase):
         self.assertIn('<b>Test</b>', output)
         add_plugin(placeholder, TextPlugin, 'en', body='<b>Test2</b>')
         request = RequestFactory().get('/?preview')
+        request.current_page = page
+        request.user = user
         context = RequestContext(request, {'page': page, 'slot': placeholder.slot})
         with self.assertNumQueries(4):
             output = template.render(context)
-        self.assertIn('<b>Test2</b>', output)
\ No newline at end of file
+        self.assertIn('<b>Test2</b>', output)
+
+    def test_render_plugin(self):
+        from django.core.cache import cache
+        cache.clear()
+        page = create_page('Test', 'col_two.html', 'en', published=True)
+        placeholder = page.placeholders.all()[0]
+        plugin = add_plugin(placeholder, TextPlugin, 'en', body='<b>Test</b>')
+        template = Template(
+            "{% load cms_tags %}{% render_plugin plugin %}")
+        request = RequestFactory().get('/')
+        user = self._create_user("admin", True, True)
+        request.user = user
+        request.current_page = page
+        request.session = {}
+        request.toolbar = CMSToolbar(request)
+        context = RequestContext(request, {'plugin': plugin})
+        with self.assertNumQueries(0):
+            output = template.render(context)
+        self.assertIn('<b>Test</b>', output)
+
+    def test_render_placeholder_with_no_page(self):
+        page = create_page('Test', 'col_two.html', 'en', published=True)
+        template = Template(
+            "{% load cms_tags %}{% placeholder test or %}< --- empty --->{% endplaceholder %}")
+        request = RequestFactory().get('/asdadsaasd/')
+        user = self.get_superuser()
+        request.user = user
+        request.current_page = page
+        request.session = {}
+        request.toolbar = CMSToolbar(request)
+        request.toolbar.edit_mode = True
+        request.toolbar.is_staff = True
+        context = RequestContext(request)
+        with self.assertNumQueries(4):
+            template.render(context)
+
