@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from functools import update_wrapper
 import pprint
-import urlparse
+from cms.utils.compat.urls import urljoin
 from cms import constants
 from cms.exceptions import CMSDeprecationWarning
 from django.conf import settings
@@ -38,26 +38,24 @@ DEFAULTS = {
     'RAW_ID_USERS': False,
     'PUBLIC_FOR': 'all',
     'CONTENT_CACHE_DURATION': 60,
-    'SHOW_START_DATE': False,
-    'SHOW_END_DATE': False,
-    'URL_OVERWRITE': True,
-    'MENU_TITLE_OVERWRITE': False,
-    'REDIRECTS': False,
-    'SEO_FIELDS': False,
     'APPHOOKS': [],
-    'SOFTROOT': False,
+    'TOOLBARS': [],
     'SITE_CHOICES_CACHE_KEY': 'CMS:site_choices',
     'PAGE_CHOICES_CACHE_KEY': 'CMS:page_choices',
     'MEDIA_PATH': 'cms/',
     'PAGE_MEDIA_PATH': 'cms_page_media/',
     'TITLE_CHARACTER': '+',
+    'PAGE_CACHE': True,
+    'PLACEHOLDER_CACHE': True,
+    'PLUGIN_CACHE': True,
     'CACHE_PREFIX': 'cms-',
     'PLUGIN_PROCESSORS': [],
     'PLUGIN_CONTEXT_PROCESSORS': [],
     'UNIHANDECODE_VERSION': None,
     'UNIHANDECODE_DECODERS': ['ja', 'zh', 'kr', 'vn', 'diacritic'],
     'UNIHANDECODE_DEFAULT_DECODER': 'diacritic',
-    'MAX_PAGE_PUBLISH_REVERSIONS': 25,
+    'MAX_PAGE_PUBLISH_REVERSIONS': 10,
+    'MAX_PAGE_HISTORY_REVERSIONS': 15,
 }
 
 
@@ -74,14 +72,9 @@ def get_media_root():
     return os.path.join(settings.MEDIA_ROOT, get_cms_setting('MEDIA_PATH'))
 
 
-@default('CMS_MEDIA_ROOT')
+@default('CMS_MEDIA_URL')
 def get_media_url():
-    return urlparse.urljoin(settings.MEDIA_URL, get_cms_setting('MEDIA_PATH'))
-
-
-@default('PLACEHOLDER_FRONTEND_EDITING')
-def get_placeholder_frontend_editing():
-    return True
+    return urljoin(settings.MEDIA_URL, get_cms_setting('MEDIA_PATH'))
 
 
 def get_templates():
@@ -102,7 +95,7 @@ def _ensure_languages_settings_new(languages):
 
     for key in defaults:
         if key not in valid_language_keys:
-            raise ImproperlyConfigured("CMS_LANGUAGES has an invalid property in the default properties: s" % key)
+            raise ImproperlyConfigured("CMS_LANGUAGES has an invalid property in the default properties: %s" % key)
 
     for key in simple_defaults:
         if key not in defaults:
@@ -199,9 +192,17 @@ def _ensure_languages_settings(languages):
 
 
 def get_languages():
+    if not isinstance(settings.SITE_ID, int):
+        raise ImproperlyConfigured(
+            "SITE_ID must be an integer"
+        )
     if not settings.USE_I18N:
         return _ensure_languages_settings(
             {settings.SITE_ID: [{'code': settings.LANGUAGE_CODE, 'name': settings.LANGUAGE_CODE}]})
+    if not settings.LANGUAGE_CODE in dict(settings.LANGUAGES):
+        raise ImproperlyConfigured(
+                        'LANGUAGE_CODE "%s" must have a matching entry in LANGUAGES' % settings.LANGUAGE_CODE
+                    )
     languages = getattr(settings, 'CMS_LANGUAGES', {
         settings.SITE_ID: [{'code': code, 'name': _(name)} for code, name in settings.LANGUAGES]
     })
@@ -225,7 +226,6 @@ COMPLEX = {
     'MEDIA_ROOT': get_media_root,
     'MEDIA_URL': get_media_url,
     # complex because not prefixed by CMS_
-    'PLACEHOLDER_FRONTEND_EDITING': get_placeholder_frontend_editing,
     'TEMPLATES': get_templates,
     'LANGUAGES': get_languages,
     'UNIHANDECODE_HOST': get_unihandecode_host,
@@ -237,3 +237,14 @@ def get_cms_setting(name):
         return COMPLEX[name]()
     else:
         return getattr(settings, 'CMS_%s' % name, DEFAULTS[name])
+
+
+def get_site_id(site):
+    from django.contrib.sites.models import Site
+    if isinstance(site, Site):
+        return site.id
+    try:
+        return int(site)
+    except (TypeError, ValueError):
+        pass
+    return settings.SITE_ID
